
import java.awt.*;
import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * crea il socket client per spiare le informazioni
 */
public class clientThreadSpy extends Thread {


   /** istanza della class ServerThread */
   private ServerThreadSpy srvManager = null;
   /** informazioni sul proxy da utlizzare */
   private ProxySessionInfo proxySession = null;
   /** connesione del client */
   private Socket socketClient = null;
   /** connessione verso il proxy */
   private Socket socketProxy = null;

   private BufferedReader inClient = null;
   private BufferedWriter outClient = null;

   private BufferedReader inProxy = null;
   private BufferedWriter outProxy = null;


   /**
    * costruttore della classe clientthreadSpy
    * @param srvManager e' l'istanza della class ServerThread
    * @param proxySession contiene le informazioni sul proxy da utlizzare 
    * @param socketClient e' il client che effettua la richiesta 
    */
   public clientThreadSpy(ServerThreadSpy srvManager,ProxySessionInfo proxySession,Socket socketClient) {
      this.srvManager   = srvManager;
      this.proxySession = proxySession;
      this.socketClient = socketClient;
      try {
         initConnection();
      } catch (Exception e) {
         e.printStackTrace();
      }

   }



   /**
    * inizializza i canali di comunicazione
    *@exception genera un IOException se non riesce a trovare il proxy
    */
   private void initConnection() throws Exception {
      socketProxy = new Socket(proxySession.getProxy(),proxySession.getPort());
      //socketClient.setSoTimeout(2000);
      inProxy  = new BufferedReader( new InputStreamReader(socketProxy.getInputStream()));
      outProxy = new BufferedWriter( new OutputStreamWriter(socketProxy.getOutputStream()));

      inClient   = new BufferedReader( new InputStreamReader(socketClient.getInputStream()));
      outClient  = new BufferedWriter( new OutputStreamWriter(socketClient.getOutputStream()));
   }





   public void run() {

      String sTemp = readClientSocketInfo(inClient);
      srvManager.inputRequest(sTemp);
      writeSocketInfo(outProxy,sTemp);
      sTemp = readServerSocketInfo(inProxy);
      srvManager.outputRequest(sTemp);
      writeSocketInfo(outClient,sTemp);
      terminate();
         
   }






   /**
    * legge le informazioni dal socket del client
    */
   private String readClientSocketInfo(BufferedReader inRead) {
      String strTemp = "";
      Vector strVector = new Vector();
      try {
         do {
            strVector.addElement(inRead.readLine());
         } while ( ((String)strVector.lastElement()).length() != 0);

      } catch (IOException e) {
         System.out.println(e.getMessage());
      }

      for (int ndx = 0; ndx != strVector.size(); ndx++) {
         strTemp =strTemp + (String)strVector.elementAt(ndx) + "\n";
      }
      return strTemp;
   } 


   /**
    * legge le informazioni dal socket
    */
   private String readServerSocketInfo(BufferedReader inRead) {
      String sTemp = "";
      try {
         while (true) {
            char ch =(char)inRead.read();
            if ( (int)ch != 65535 ) {
               sTemp += ch;
            } else {
               return sTemp;
            }
         }
      } catch (IOException e) {
         System.out.println(e.getMessage());
      }
      return sTemp;
   }


   /**
    * scrive le informazioni sul socket 
    */
   private void writeSocketInfo(BufferedWriter outWrite,String sTemp) {
      try {
         outWrite.write(sTemp,0,sTemp.length());
         outWrite.flush();
      } catch (IOException e) {
         System.out.println(e.getMessage());
      }
   }


   private void terminate() {
      try {
         socketClient.close();
         socketProxy.close();
      } catch (IOException e) {
         
      }
   }

}

